package tools.descartes.dql.connector.kieker.tests;

import static org.junit.Assert.*;

import java.util.List;
import java.util.Map;

import org.eclipse.xtext.junit4.InjectWith;
import org.eclipse.xtext.junit4.XtextRunner;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;

import tools.descartes.dql.connector.kieker.KiekerHelper;
import tools.descartes.dql.connector.kieker.KiekerManager;
import tools.descartes.dql.connector.kieker.structures.Component;
import tools.descartes.dql.connector.kieker.structures.Trace;
import tools.descartes.dql.connector.kieker.structures.records.EventRecord;
import tools.descartes.dql.connector.kieker.structures.records.ResourceRecord;
import tools.descartes.dql.lang.DescartesQLInjectorProvider;
import tools.descartes.dql.models.mapping.domain.DomainFactory;
import tools.descartes.dql.models.mapping.domain.Entity;
import tools.descartes.dql.models.mapping.mapping.EntityMapping;
import tools.descartes.dql.models.mapping.mapping.MappingFactory;

@RunWith(XtextRunner.class)
@InjectWith(DescartesQLInjectorProvider.class)
public class KiekerManagerTest {

	
	EntityMapping request = MappingFactory.eINSTANCE.createEntityMapping();
	String pathToConfig = "dql.properties";
	
	
	
	@Test
	public void testListEntitiesAnalysis(){
		this.request.setModelLocation(pathToConfig);
		KiekerManager manager = new KiekerManager(request);
		manager.run();
		Trace result = manager.getAnalysisResult();
		assertNotNull(result);
		assertTrue(result.getResources().size() > 0);
		assertTrue(result.getEvents().size() > 0);
	}

	
	@Test
	public void testUtilizationAnalysis(){
		Entity cpu1 = DomainFactory.eINSTANCE.createEntity();
		cpu1.setEntityType("RESOURCE");
		cpu1.setAlias("res1");
		cpu1.setIdentifier("cpu1");
		Entity cpu2 = DomainFactory.eINSTANCE.createEntity();
		cpu2.setEntityType("RESOURCE");
		cpu2.setAlias("res2");
		cpu2.setIdentifier("cpu2");
		this.request.setModelLocation(pathToConfig);
		this.request.getEntities().add(cpu1);
		this.request.getEntities().add(cpu2);
		
		KiekerManager manager = new KiekerManager(request);
		manager.run();
		Trace result = manager.getAnalysisResult();
		assertNotNull(result);
		List<ResourceRecord> resources =result.getResources();
		assertTrue(resources.size() > 0);
		assertTrue(result.getResourceIdentifierToResourceMap().size() > 0);
		assertEquals(result.getResourceIdentifierToResourceMap().size(),this.request.getEntities().size());
		int recordCounter = 0;
		for (ResourceRecord record : resources) {
			if (record.getIdentifier().getRessourceName().equals(cpu1.getIdentifier())){
				recordCounter++;
			}
		}
		assertTrue(recordCounter > 0);
		recordCounter = 0;
		for (ResourceRecord record : resources) {
			if (record.getIdentifier().getRessourceName().equals(cpu2.getIdentifier())){
				recordCounter++;
			}
		}
		assertTrue(recordCounter > 0);
		
	}
	
	@Test
	public void testResponseTimeAnalysis(){
		Entity res1 = DomainFactory.eINSTANCE.createEntity();
		res1.setEntityType("SERVICE");
		res1.setAlias("svc1");
		res1.setIdentifier("org.mybatis.jpetstore.web.actions.CatalogActionBean.getItem()");
		this.request.setModelLocation(pathToConfig);
		this.request.getEntities().add(res1);
		
		KiekerManager manager = new KiekerManager(request);
		manager.run();
		Trace result = manager.getAnalysisResult();
		assertNotNull(result);
		Map<Component, Trace> services = result.getComponentToTraceMap();
		assertTrue(services.size() > 0);
		Component comp = new Component(KiekerHelper.getComponentString(res1.getIdentifier()));
		Trace tr = services.get(comp);
		assertNotNull(tr);
		assertTrue(tr.getEvents().size() > 0);
		assertTrue(result.getComponentIdentifierToComponentMap().size() > 0);
		assertEquals(result.getComponentIdentifierToComponentMap().size(),this.request.getEntities().size());
		int recordCounter = 0;
		for (EventRecord er : tr.getEvents()) {
			String dqlId = res1.getIdentifier();
			
			String kiekerId = er.getIdentifier().getComponentID();
			if (er.getIdentifier().isInterfaceIdentifier()){
				kiekerId += "." + er.getIdentifier().getInterfaceID();
			}
			if (dqlId.equals(kiekerId)) {
				recordCounter++;
			}
		}
		assertTrue(recordCounter > 0);
		
	}
	
	
}
