package tools.descartes.dql.connector.kieker.filter;

import kieker.analysis.IProjectContext;
import kieker.analysis.plugin.annotation.InputPort;
import kieker.analysis.plugin.annotation.OutputPort;
import kieker.analysis.plugin.annotation.Plugin;
import kieker.analysis.plugin.filter.AbstractFilterPlugin;
import kieker.common.configuration.Configuration;
import kieker.common.record.IMonitoringRecord;
import kieker.common.record.system.CPUUtilizationRecord;
import tools.descartes.dql.connector.kieker.structures.cpu.CpuMap;
import tools.descartes.dql.connector.kieker.structures.cpu.CpuMapKey;
import tools.descartes.dql.connector.kieker.structures.cpu.CpuMapValue;

@Plugin(name = "CPU utilization filter", description = "Extracts CPU-utilization-information from incoming kieker monitoring records", outputPorts = {
		@OutputPort(name = UtilizationFilter.OUTPUT_PORT_NAME, description = "Outputs CPUUtilRecords", eventTypes = {
				IMonitoringRecord.class }),
		@OutputPort(name = UtilizationFilter.OUTPUT_PORT_NAME_OTHER, description = "Outputs untouched information", eventTypes = {
				IMonitoringRecord.class }) })

public class UtilizationFilter extends AbstractFilterPlugin {

	public static final String INPUT_PORT_NAME = "newMonitoringRecord";
	public static final String OUTPUT_PORT_NAME = "forwardingResUtRecord";
	public static final String OUTPUT_PORT_NAME_OTHER = "untouchedIMonitoringRecords";

	public UtilizationFilter(final Configuration configuration, final IProjectContext projectContext) {
		super(configuration, projectContext);
	}

	@InputPort(name = UtilizationFilter.INPUT_PORT_NAME, description = "Extract resource-information from monitoring record", eventTypes = {
			IMonitoringRecord.class })

	public void runFilter(final Object record) {
		if (record instanceof CPUUtilizationRecord) {
			final CPUUtilizationRecord cpuUtRecord = (CPUUtilizationRecord) record;

			final long timestamp = cpuUtRecord.getLoggingTimestamp();
			final String cpuID = cpuUtRecord.getCpuID();
			final String hostName = cpuUtRecord.getHostname();
			final Double system = cpuUtRecord.getSystem();
			final Double user = cpuUtRecord.getUser();

			final long interval = cpuUtRecord.getLoggingTimestamp() - cpuUtRecord.getTimestamp();
			final Double totalUtilization = cpuUtRecord.getTotalUtilization();
			final Double nice = cpuUtRecord.getNice();
			final Double idle = cpuUtRecord.getIdle();
			final Double wait = cpuUtRecord.getWait();
			final Double irq = cpuUtRecord.getIrq();
			
			CpuMapKey key = new CpuMapKey(cpuID, hostName, user, system);
			CpuMapValue value = new CpuMapValue(timestamp, interval, totalUtilization, nice, idle, wait, irq);
			CpuMap.getInstance().put(key, value);

			super.deliver(OUTPUT_PORT_NAME, cpuUtRecord);
		} else {
			super.deliver(OUTPUT_PORT_NAME_OTHER, record);
		}
	}

	@Override
	public Configuration getCurrentConfiguration() {
		return new Configuration();
	}
}
